@@ -1,38 +1,37 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from decimal import Decimal
 from collections import defaultdict
+from decimal import Decimal
 from itertools import groupby
 
 from sql import Literal
 
-from trytond.model import ModelView, ModelSQL, fields, Check
-from trytond.wizard import Wizard, StateAction
+from trytond.model import Check, Index, ModelSQL, ModelView, dualmethod, fields
+from trytond.modules.currency.fields import Monetary
+from trytond.pool import Pool, PoolMeta
 from trytond.pyson import Eval, PYSONEncoder
 from trytond.transaction import Transaction
-from trytond.pool import Pool, PoolMeta
+from trytond.wizard import StateAction, Wizard
 
 
 class Line(ModelSQL, ModelView):
     'Analytic Line'
     __name__ = 'analytic_account.line'
-    debit = fields.Numeric('Debit', digits=(16, Eval('currency_digits', 2)),
-        required=True, depends=['currency_digits'])
-    credit = fields.Numeric('Credit', digits=(16, Eval('currency_digits', 2)),
-        required=True, depends=['currency_digits'])
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'on_change_with_currency_digits')
+    debit = Monetary(
+        "Debit", currency='currency', digits='currency', required=True)
+    credit = Monetary(
+        "Credit", currency='currency', digits='currency', required=True)
+    currency = fields.Function(fields.Many2One(
+            'currency.currency', "Currency"),
+        'on_change_with_currency')
     company = fields.Function(fields.Many2One('company.company', 'Company'),
         'on_change_with_company', searcher='search_company')
-    account = fields.Many2One('analytic_account.account', 'Account',
-        required=True, select=True, domain=[
-            ('type', 'not in', ['view', 'distribution']),
-            ['OR',
-                ('company', '=', None),
-                ('company', '=', Eval('company', -1)),
-                ],
-            ],
-        depends=['company'])
+    account = fields.Many2One(
+        'analytic_account.account', "Account", required=True,
+        domain=[
+            ('type', '=', 'normal'),
+            ('company', '=', Eval('company', -1)),
+            ])
     move_line = fields.Many2One('account.move.line', 'Account Move Line',
             ondelete='CASCADE', required=True)
     date = fields.Date('Date', required=True)
@@ -46,6 +45,10 @@ class Line(ModelSQL, ModelView):
                 Check(t, t.credit * t.debit == 0),
                 'account.msg_line_debit_credit'),
             ]
+        cls._sql_indexes.update({
+                Index(t, (t.account, Index.Equality())),
+                Index(t, (t.date, Index.Range())),
+                })
         cls._order.insert(0, ('date', 'ASC'))
 
     @classmethod
@@ -53,10 +56,6 @@ class Line(ModelSQL, ModelView):
         super(Line, cls).__register__(module_name)
         table = cls.__table_handler__(module_name)
 
-        # Migration from 4.0: remove name and journal
-        for field_name in ['name', 'journal']:
-            table.not_null_action(field_name, action='remove')
-
         # Migration from 5.0: replace credit_debit constraint by credit_debit_
         table.drop_constraint('credit_debit')
 
@@ -74,15 +73,14 @@ class Line(ModelSQL, ModelView):
         return Decimal(0)
 
     @fields.depends('move_line', '_parent_move_line.account')
-    def on_change_with_currency_digits(self, name=None):
+    def on_change_with_currency(self, name=None):
         if self.move_line and self.move_line.account:
-            return self.move_line.account.company.currency.digits
-        return 2
+            return self.move_line.account.company.currency
 
     @fields.depends('move_line', '_parent_move_line.account')
     def on_change_with_company(self, name=None):
         if self.move_line and self.move_line.account:
-            return self.move_line.account.company.id
+            return self.move_line.account.company
 
     @classmethod
     def search_company(cls, name, clause):
@@ -142,7 +140,7 @@ class Line(ModelSQL, ModelView):
 class Move(metaclass=PoolMeta):
     __name__ = 'account.move'
 
-    @classmethod
+    @dualmethod
     @ModelView.button
     def post(cls, moves):
         pool = Pool()
@@ -153,29 +151,29 @@ class Move(metaclass=PoolMeta):
         MoveLine.set_analytic_state(lines)
         MoveLine.save(lines)
 
-    def cancel(self, default=None):
-        'Reverse credit/debit of analytic lines'
-        pool = Pool()
-        AnalyticLine = pool.get('analytic_account.line')
-        cancel_move = super(Move, self).cancel(default)
-        analytic_lines = []
-        for line in cancel_move.lines:
-            for analytic_line in line.analytic_lines:
-                analytic_line.debit, analytic_line.credit = (
-                    analytic_line.credit, analytic_line.debit)
-                analytic_lines.append(analytic_line)
-        AnalyticLine.save(analytic_lines)
-        return cancel_move
+    def _cancel_default(self, reversal=False):
+        default = super()._cancel_default(reversal=reversal)
+        if reversal:
+            default['lines.analytic_lines.debit'] = (
+                lambda data: data['credit'])
+            default['lines.analytic_lines.credit'] = (
+                lambda data: data['debit'])
+        else:
+            default['lines.analytic_lines.debit'] = (
+                lambda data: data['debit'] * -1)
+            default['lines.analytic_lines.credit'] = (
+                lambda data: data['credit'] * -1)
+        return default
 
 
-class MoveLine(ModelSQL, ModelView):
+class MoveLine(metaclass=PoolMeta):
     __name__ = 'account.move.line'
     analytic_lines = fields.One2Many('analytic_account.line', 'move_line',
             'Analytic Lines')
     analytic_state = fields.Selection([
             ('draft', 'Draft'),
             ('valid', 'Valid'),
-            ], 'Analytic State', readonly=True, select=True)
+            ], "Analytic State", readonly=True, sort=False)
 
     @classmethod
     def __setup__(cls):
@@ -188,21 +186,6 @@ class MoveLine(ModelSQL, ModelView):
             })
         cls._check_modify_exclude |= {'analytic_lines', 'analytic_state'}
 
-    @classmethod
-    def __register__(cls, module_name):
-        table = cls.__table_handler__(module_name)
-        cursor = Transaction().connection.cursor()
-        sql_table = cls.__table__()
-
-        state_exist = table.column_exist('analytic_state')
-
-        super(MoveLine, cls).__register__(module_name)
-
-        # Migration from 4.0: add analytic_state
-        if not state_exist:
-            cursor.execute(
-                *sql_table.update([sql_table.analytic_state], ['valid']))
-
     @classmethod
     def default_analytic_state(cls):
         return 'draft'
